gitignore = """
# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
share/python-wheels/
*.egg-info/
.installed.cfg
*.egg
MANIFEST

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.nox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.cover
*.py,cover
.hypothesis/
.pytest_cache/
cover/

# Translations
*.mo
*.pot

# Django stuff:
*.log
local_settings.py
db.sqlite3
db.sqlite3-journal

# Flask stuff:
instance/
.webassets-cache

# Scrapy stuff:
.scrapy

# Sphinx documentation
docs/_build/

# PyBuilder
.pybuilder/
target/

# Jupyter Notebook
.ipynb_checkpoints

# IPython
profile_default/
ipython_config.py

# pyenv
#   For a library or package, you might want to ignore these files since the code is
#   intended to run in multiple environments; otherwise, check them in:
# .python-version

# pipenv
#   According to pypa/pipenv#598, it is recommended to include Pipfile.lock in version control.
#   However, in case of collaboration, if having platform-specific dependencies or dependencies
#   having no cross-platform support, pipenv may install dependencies that don't work, or not
#   install all needed dependencies.
#Pipfile.lock

# poetry
#   Similar to Pipfile.lock, it is generally recommended to include poetry.lock in version control.
#   This is especially recommended for binary packages to ensure reproducibility, and is more
#   commonly ignored for libraries.
#   https://python-poetry.org/docs/basic-usage/#commit-your-poetrylock-file-to-version-control
#poetry.lock

# PEP 582; used by e.g. github.com/David-OConnor/pyflow
__pypackages__/

# Celery stuff
celerybeat-schedule
celerybeat.pid

# SageMath parsed files
*.sage.py

# Environments
.env
.venv
env/
venv/
ENV/
env.bak/
venv.bak/

# Spyder project settings
.spyderproject
.spyproject

# Rope project settings
.ropeproject

# mkdocs documentation
/site

# mypy
.mypy_cache/
.dmypy.json
dmypy.json

# Pyre type checker
.pyre/

# pytype static type analyzer
.pytype/

# Cython debug symbols
cython_debug/

# PyCharm
#  JetBrains specific template is maintainted in a separate JetBrains.gitignore that can
#  be found at https://github.com/github/gitignore/blob/main/Global/JetBrains.gitignore
#  and can be added to the global gitignore or merged into this file.  For a more nuclear
#  option (not recommended) you can uncomment the following to ignore the entire idea folder.
#.idea/
"""


import os
import sys
import toml
import fire
import subprocess

from choam.create_setup_file import create_setup_file
from choam.folder_structure import FolderStructure as FS
from choam.gitignore import gitignore

class Choam:
  def __init__(self):
    pass

  def _get_config():
    with open(f"{os.getcwd()}/Choam.toml", "r") as f:
      return toml.loads(
        f.read()
      )
    
  def _set_config(content: str):
    with open(f"{os.getcwd()}/Choam.toml", "w") as f:
      f.write(content)
  
  def _log(message: str):
    print(f"\n\t{message}")
    
  def _log_multiple(messages: "list[str]"):
    print()
    for message in messages:
      print(f'\t{message}')
  
  def init(self, name: str):
    '''
    Initalize a new Choam project in the working 
    directory
    '''

    directory = os.getcwd()
    
    if FS.is_choam_project(directory):
      Choam._log("Already a Choam project.")
      return
    
    template = {
      f"\\{name}\\__main__.py": "",
      f"\\{name}\\__init__.py": "__version__ == '0.1'",
      f"Choam.toml": f'[package]\nname = "{name}\nversion = "0.0.1"\ndescription = ""',
      f"README.md": f"# {name}\n#### This project was constructed with [Choam](https://github.com/cowboycodr/choam)",
      f".gitignore": gitignore
    }
    
    FS.construct_from_dict(template, directory)
  
  def new(self, name: str):
    '''
    Create a new directory and initalize a Choam project
    inside of it
    '''

    directory = os.getcwd()
      
    folder_name = name.lower()
    
    # Choam project template
    template = {
      f"\\{folder_name}\\{folder_name}\\__main__.py": "",
      f"\\{folder_name}\\{folder_name}\\__init__.py": "__version__ == '0.1'",
      f"\\{folder_name}\\Choam.toml": f'[package]\nname = "{name}"\nversion = "0.0.1"\ndescription = ""',
      f"\\{folder_name}\\README.md": f"# {name}\n#### This project was constructed with [Choam](https://github.com/cowboycodr/choam)",
      f"\\{folder_name}\\.gitignore": gitignore
    }
    
    FS.construct_from_dict(template, directory)
    
  def run(self):
    '''
    Run choam project main file
    '''
    
    if not FS.is_choam_project():
      Choam._log("Not a Choam project.")
      return
    
    folder_name = Choam._get_config()['package']['name'].lower()
    
    os.system(
      f"python -m {folder_name}"
    )
    
  def setup(self):
    directory = os.getcwd()
    
    configs = Choam._get_config()
    
    package_config = configs['package']
    
    name = package_config['name']
    version = package_config['version']
    
    try:
      description = package_config['description']
    except:
      description = ""
    
    try:
      repo_url = package_config['repo']
    except:
      repo_url = ''
    
    try:
      keywords = package_config['keywords']
    except:
      keywords = []
      
    modules = configs['modules']
    
    template = {
      f"\\setup.py": create_setup_file(
        name,
        version,
        description,
        keywords,
        modules,
        repo_url
      )
    }
    
    FS.construct_from_dict(template, f"{directory}\\")
    
    Choam._log_multiple(
      [
        f"Successfully setup '{name}' for PyPi publication",
        f"Use '$ choam publish' when configurations have been set"
      ]
    )
    
  def add(self, dependency_name: str):
    config = Choam._get_config()

    config['modules'][dependency_name] = "*"
    
    Choam._set_config(toml.dumps(config))

  def install(self):
    config = Choam._get_config()
    modules = config['modules']

    for mod in modules:
      Choam._log(f"Installing {mod}")

      module_string = f"{mod}=={modules[mod]}" if modules[mod] != "*" else f"{mod}--upgrade"
      upgrade_module = module_string.endswith('--upgrade')

      subprocess.call([sys.executable, "-m", "pip", "install", module_string.replace("--upgrade", ""), "--upgrade" if upgrade_module else ""])
    
  def publish(self):
    self.add("twine")
    self.install()

    Choam._log("Attempting real publication to https://test.pypi.org/legacy")
    subprocess.call([sys.executable, "-m", "setup.py", "sdist", "bdist_wheel"])
    subprocess.call([sys.executable, "-m", "twine", "upload", f"--repository-url", "https://test.pypi.org/legacy/", "dist/*"])

    Choam._log_multiple([
      "Test publication successful",
      "Attempting real publication..."
    ])

    subprocess.call([sys.executable, "-m", "twine", "upload", "dist/*"])

    Choam._log("Real publication successful")
  
if __name__ == '__main__':
  fire.Fire(Choam())


import os

class FolderStructure:
  '''
  `Choam's` folder structure consturction/destruction
  manager class
  '''
  
  @staticmethod
  def is_choam_project(_dir: "str | None" = None) -> bool:
    '''
    Returns true if the directory passed in 
    is a Choam project
    '''
    
    if not _dir:
      _dir = os.getcwd()
    
    if not os.path.exists(_dir):
      return
    
    return ("Choam.toml" in os.listdir(os.path.abspath(_dir)))

  def is_publishable(_dir: "str | None" = None):
    ''''
    Returns True if the directory fits 
    Choam's publication requirements
    '''

    if not _dir:
      _dir = os.getcwd()

    if not os.path.exists(_dir):
      return

    return ("setup.py" in os.listdir(os.path.abspath(_dir)))
  
  @staticmethod
  def _create_file(filepath: str, content: "str | None") -> str:
    filepath = os.path.abspath(filepath)
    filepath = filepath.replace("\\\\", '').replace('\\', '/')

    file_name = filepath.split("/")[-1]
    folder_path = filepath.replace(file_name, '')
    
    print(file_name, folder_path)

    if not os.path.exists(folder_path):
      os.makedirs(folder_path)
      
    with open(filepath, "w") as file:
      file.write(content if content else "")
      
    return filepath
  
  @staticmethod
  def construct_from_dict(_dict: dict, output_dir: str):
    '''
    Construct folder structure from dictionary
    '''
    
    output_dir = os.path.abspath(output_dir)
    
    for file_name in _dict.keys():
      FolderStructure._create_file(f"{output_dir}\\{file_name}", _dict[file_name])


from setuptools import setup, find_packages

setup(
    name="Choam",
    version="0.0.2",
    description="Python project scaffolder/manager",
    packages=find_packages(),
    keywords=['package', 'manager'],
    install_requires=['toml', 'twine', 'fire'],
    project_urls={
        'Source': 'https://github.com/cowboycodr/choam'
    },
)


def create_setup_file(
  name: str, 
  version: str, 
  description: str, 
  keywords: "list[str]", 
  modules: "dict[str]",
  repo_url: str
  ):
  '''
  Generate basic setup.py file contents
  
  Package lists are currently generated automatically
  
  Arguments:
    @name - project name
    @version - current version
  '''
  
  return '\n'.join([
    'from setuptools import setup, find_packages',
    '',
    'setup(',
    f'    name="{name}",',
    f'    version="{version}",',
    f'    description="{description}",',
    f'    packages=find_packages(),',
    f'    keywords={keywords},',
    f'    install_requires={[key for key in modules.keys()]},',
    '    project_urls={',
    f"        'Source': '{repo_url}'",
    "    },",
    ')'
  ])